package com.taover.ai.controller.api;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.taover.ai.util.RedisUtil;
import com.taover.util.UtilLog;
import com.taover.util.bean.ResultInfo;
import com.taover.util.bean.UtilResultInfo;

@RestController("api.demo")
@RequestMapping("/api/demo")
public class DemoController {	
	
	 @Resource
	 private RedisUtil redisUtil;
	 
	/**
	 * @apiDefine ResultInfo
	 * @apiSuccess {string}
	 * @apiSuccessExample {json} Success-Response:
	 * 	{
	 * 		code:ok,
	 * 		error:null
	 * 	}
 	 * @apiError ThrowException 出现异常
	 * @apiErrorExample 
	 * 	{
	 * 		code:fail,
	 * 		error:null exception
	 * 	}
	 */
	
	/**
	 * @api {GET} /v1/demo 查询demo列表
	 * @apiDescription 查询demo列表数据 
	 * 返回分页结果
	 * 
	 * @apiGroup Demo
	 * @apiName get列表
	 * @apiVersion 1.0.0
	 * @apiHeader Authorization
	 * 
	 * @apiUse ResultInfo
	 */
	@GetMapping
	public ResultInfo get(HttpServletRequest request){
		return UtilResultInfo.getSuccess("get success");
	}
	
	/**
	 * @api {GET} /v1/demo/{ssid} 查询ssid对应的demo
	 * @apiDescription 查询ssid对应demo数据 
	 *  
	 * @apiGroup Demo
	 * @apiName get实体数据
	 * @apiVersion 1.0.0
	 * @apiHeader Authorization
	 * 
	 * @apiUse ResultInfo
	 */
	@GetMapping("/{ssid}")
	public ResultInfo get(String ssid){
		return UtilResultInfo.getSuccess("get by ssid success");
	}
	
	/**
	 * @api {POST} /v1/demo 创建demo
	 * @apiDescription 创建demo 
	 *  
	 * @apiGroup Demo
	 * @apiName get实体数据
	 * @apiVersion 1.0.0
	 * @apiHeader Authorization
	 * 
	 * @apiUse ResultInfo
	 */
	@PostMapping
	public ResultInfo post(HttpServletRequest request){		
		return UtilResultInfo.getSuccess("post success");
	}
	
	/**
	 * @api {DELETE} /v1/demo/{ssid} 删除ssid对应的demo
	 * @apiDescription 删除ssid对应demo数据 
	 *  
	 * @apiGroup Demo
	 * @apiName 删除实体数据
	 * @apiVersion 1.0.0
	 * @apiHeader Authorization
	 * 
	 * @apiUse ResultInfo
	 */
	@DeleteMapping("/{ssid}")
	public ResultInfo delete(String ssid){
		return UtilResultInfo.getSuccess("delete success");
	}
	
	/**
	 * @api {PUT} /v1/demo/{ssid} 修改demo数据
	 * @apiDescription 修改demo数据 
	 *  
	 * @apiGroup Demo
	 * @apiName 修改实体数据
	 * @apiVersion 1.0.0
	 * @apiHeader Authorization
	 * 
	 * @apiUse ResultInfo
	 */
	@PutMapping("/{ssid}")
	public ResultInfo put(String ssid, HttpServletRequest request){
		return UtilResultInfo.getSuccess("put for ssid success");
	}
	
	@GetMapping(value="/testjson")
	public ResultInfo testJson(){
		try{
			int a = 1/0;
		}catch(Exception e){
			UtilLog.errorForException(e, this.getClass());
		}
		return UtilResultInfo.getSuccess("测试异常");
	}
	
	@GetMapping(value="/testRedis")
	public ResultInfo testRedis(){
		try{
			redisUtil.set("aaa", "test");
			String aaa = (String) redisUtil.get("aaa");
			System.out.print(aaa);
		}catch(Exception e){
			UtilLog.errorForException(e, this.getClass());
		}
		return UtilResultInfo.getSuccess("测试异常");
	}
}
