package com.taover.ai.controller.manage;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.taover.ai.service.AddressService;
import com.taover.util.bean.ResultInfo;
import com.taover.util.bean.UtilResultInfo;

@RestController("manage.address")
@RequestMapping("/manage/address")
public class AddressController {
	@Resource
	private AddressService addressService;
	/**
	 * @apiDefine ResultInfo
	 * @apiSuccess {string}
	 * @apiSuccessExample {json} Success-Response:
	 * 	{
	 * 		code:ok,
	 * 		error:null
	 * 	}
 	 * @apiError ThrowException 出现异常
	 * @apiErrorExample 
	 * 	{
	 * 		code:fail,
	 * 		error:null exception
	 * 	}
	 */	
	
	/**
	 * @api {GET} /v1/address/ 查询ssid对应的demo
	 * @apiDescription 查询ssid对应demo数据 
	 *  
	 * @apiGroup Demo
	 * @apiName get实体数据
	 * @apiVersion 1.0.0
	 * @apiHeader Authorization
	 * 
	 * @apiUse ResultInfo
	 */
	@GetMapping("/examine/pcdDetail")
	public ResultInfo examinePcdDetail(HttpServletRequest request){
		return UtilResultInfo.getSuccess("success");
	}	
	
	/**
	 * @api {POST} /v1/demo 创建demo
	 * @apiDescription 创建demo 
	 *  
	 * @apiGroup Demo
	 * @apiName get实体数据
	 * @apiVersion 1.0.0
	 * @apiHeader Authorization
	 * 
	 * @apiUse ResultInfo
	 */
	@PostMapping
	public ResultInfo post(HttpServletRequest request){
		return UtilResultInfo.getSuccess("post success");
	}
	
	/**
	 * @api {DELETE} /v1/demo/{ssid} 删除ssid对应的demo
	 * @apiDescription 删除ssid对应demo数据 
	 *  
	 * @apiGroup Demo
	 * @apiName 删除实体数据
	 * @apiVersion 1.0.0
	 * @apiHeader Authorization
	 * 
	 * @apiUse ResultInfo
	 */
	@DeleteMapping("/{ssid}")
	public ResultInfo delete(String ssid){
		return UtilResultInfo.getSuccess("delete success");
	}
	
	/**
	 * @api {PUT} /v1/demo/{ssid} 修改demo数据
	 * @apiDescription 修改demo数据 
	 *  
	 * @apiGroup Demo
	 * @apiName 修改实体数据
	 * @apiVersion 1.0.0
	 * @apiHeader Authorization
	 * 
	 * @apiUse ResultInfo
	 */
	@PutMapping("/{ssid}")
	public ResultInfo put(String ssid, HttpServletRequest request){
		return UtilResultInfo.getSuccess("put for ssid success");
	}
}
